﻿using Arction.WPF.LightningChartUltimate;
using Arction.WPF.LightningChartUltimate.SeriesXY;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfApplication2
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly double InvalidValue = -Math.Pow(10, 6);//Konstante für ungültige Werte
        private readonly double InvalidValueOvershoot = -Math.Pow(10, 6) + 1;
        private const double NonApprovedValueOffset = -50000;//Ungültige Werte werden mit diesem Offset verschoben, damit beim Zeichnen nicht nachträglich der Status wieder ermittelt werden muss
        private const double NonApprovedValueOvershoot = -20000;
        public MainWindow()
        {
            InitializeComponent();
            InitChart();
        }

        private void InitChart()
        {
            Random r = new Random();
            chart.ChartRenderOptions.MultiCoreProcessing = MultiCoreProcessing.Off;
            chart.ViewXY.XAxes.Add(new Arction.WPF.LightningChartUltimate.Axes.AxisX(chart.ViewXY));
            chart.ViewXY.YAxes.Add(new Arction.WPF.LightningChartUltimate.Axes.AxisY(chart.ViewXY));
            for (int i = 0; i < 5; i++)
            {
                PointLineSeries pls = new PointLineSeries(chart.ViewXY, chart.ViewXY.XAxes[0], chart.ViewXY.YAxes[0]);
                SeriesPoint[] points = new SeriesPoint[100];
                for (int j = 0; j < points.Length; j++)
                {
                    points[j] = new SeriesPoint(j, r.NextDouble());
                }
                pls.LineStyle.Width = 1.0f;
                pls.Points = points;
                pls.CustomLinePointColoringAndShaping += CustomLinePointColoringAndShaping;
                chart.ViewXY.PointLineSeries.Add(pls);
            }
        }

        private float _linewidth;
        private void Slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (chart == null || chart.ViewXY == null || chart.ViewXY.PointLineSeries == null)
                return;
            _linewidth = (float)((int)e.NewValue + 1.0f);
            foreach (PointLineSeries series in chart.ViewXY.PointLineSeries)
            {
                series.LineStyle.Width = (float)((int)e.NewValue + 1.0f);
            }
        }

        private void CustomLinePointColoringAndShaping(PointLineSeriesBase series, ref PointFloat[] coords, ref int[] colors, bool canModifyCoords, bool canModifyColors)
        {
            series.LineStyle.Width = 1.0f;
            if (!(canModifyColors && canModifyCoords))
            {

                Debug.WriteLine("Modify Colors: " + canModifyColors + " Modify Coords: " + canModifyCoords);

                return;
            }

            var iCount = coords.Length;
            var listOutputCoords = new List<PointFloat>();
            var listOutputColors = new List<int>();
            bool wasLow = true;
            int outputColor = (series.LineStyle.Color.A << 24) | (series.LineStyle.Color.R << 16) | (series.LineStyle.Color.G << 8) | series.LineStyle.Color.B; ;
            for (var i = 0; i < iCount; i++)
            {

                double yValue = 0;
                chart.ViewXY.YAxes[0].CoordToValue(coords[i].Y, out yValue);


                outputColor = (series.LineStyle.Color.A << 24) | (series.LineStyle.Color.R << 16) | (series.LineStyle.Color.G << 8) | series.LineStyle.Color.B; ;
                if (yValue > InvalidValueOvershoot && yValue < NonApprovedValueOvershoot)
                {
                    outputColor = System.Drawing.Color.LightYellow.ToArgb();
                    coords[i].Y = chart.ViewXY.YAxes[0].ValueToCoord(yValue + -NonApprovedValueOffset);
                }

                if (yValue < InvalidValueOvershoot)
                {
                    if (!wasLow && i > 0)
                    {//Vorgänger muss noch einmal transparent gezeichnet werden
                        listOutputColors.Add(System.Drawing.Color.Transparent.ToArgb());
                        listOutputCoords.Add(coords[i - 1]);
                    }
                    listOutputColors.Add(System.Drawing.Color.Transparent.ToArgb());
                    wasLow = true;
                }
                else
                {
                    if (wasLow)//Aktueller Wert muss noch einmal transparent gezeichnet werden
                    {
                        listOutputColors.Add(System.Drawing.Color.Transparent.ToArgb());
                        listOutputCoords.Add(coords[i]);
                    }
                    listOutputColors.Add(outputColor);
                    wasLow = false;
                }

                listOutputCoords.Add(coords[i]);
            }

            coords = listOutputCoords.ToArray();
            colors = listOutputColors.ToArray();
            series.LineStyle.Width = _linewidth;
        }

    }
}
